function [upperinvA] = imatinvupper(iA)
%BEGINDOC=================================================================
% .Author.
%
%  Jaroslav Horacek
%
%-------------------------------------------------------------------------
% .Description.
%
%   computes upper bound to inversion matrix of iA, works if iA is H-matrix
%
%-------------------------------------------------------------------------
% .Input parameters.
%   
%   iA ...  interval or real matrix 
%
%------------------------------------------------------------------------
% .Output parameters.
%
%   upperinvA ...  inverse matrix
%
%------------------------------------------------------------------------
% .Implementation details. 
%
%  Uses algorithm described in Neumaier, A simple derivation of 
%  Hansen-Bliek-Rohn-Ning-Kearfott enclosure for linear interval equations, 1999
%
%------------------------------------------------------------------------
% .Licence.
%  
%   Copyright (C) 2018  Charles University in Prague, Czech Republic
%
%   LIME 1.0 is free for private use and for purely academic purposes. 
%   It would be very kind from the future user of LIME 1.0 to give 
%   reference that this software package has been developed 
%   by at Charles University, Czech Republic.
%
%   For any other use of LIME 1.0 a license is required.
%
%   THIS SOFTWARE IS PROVIDED AS IS AND WITHOUT ANY EXPRESS OR IMPLIED 
%   WARRANTIES, INCLUDING, WITHOUT LIMITATIONS, THE IMPLIED WARRANTIES 
%   OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.    
%    
%-------------------------------------------------------------------------
% .History.
%       
%   2018-10-02    first version
%
%------------------------------------------------------------------------
% .Todo.
%   
%
%ENDDOC===================================================================


invA = [];

if (m != n)
  error('imatinvupper: iA must be n x n');
end

diagiA = diag(iA);
compA = imatcomp(iA);

B = inv(A);  % approximatate of <A> inverse
v = abs(B*ones(n,1)); % approximate solution of <A> x = e


% enforce nonnegativity of v
SetRoundDown;
u = A*v;
% is approximately equal to all-one vector
if ~all(min(u)>0),
% check positivity of u
% A is numerically not an H-matrix
x = midrad(0,inf+zeros(n,1));
else
dAc = diag(A); % save diagonal entries before overwriting A
A = A*B-eye(n);
% A now contains the residual matrix
SetRoundUp;
w = zeros(1,n);
for i=1:n,
w = max(w,(-A(i,:))/u(i));
end;
B = B+v*w;
% rigorous upper bound for exact B
u = B*abs(b);
d = diag(B);
alpha = dAc+(-1)./d;
% ensures upward rounding
beta = u./d-abs(b);
x = (b+midrad(0,beta))./(dA+midrad(0,alpha));
end;

end