function [id, state, method] = idet (iA)
%BEGINDOC=================================================================
% .Author.
%
%  Jaroslav Horacek
%
%-------------------------------------------------------------------------
% .Description.
%
%   Returns an interval enclosure of a determinant. For matrices of order 1,2
%   returns the hull. Simple general (not necessarily the fastest) method.
%
%
%-------------------------------------------------------------------------
% .Input parameters.
%   
%   iA ... a square interval matrix
%
%------------------------------------------------------------------------
% .Output parameters.
%
%   id ... interval enclosure of determinant
%   state ... 'vec' - finite enclosure, 'inf' - infinite enclosure, 'hull' -
%             the method returns the hull  
%   method ... which method was used; 0-small matrix, 1-Cramer, 2-GE, 
%               3-intersection of Gerschgorin and Hadamard's inequality
%
%------------------------------------------------------------------------
% .Implementation details. 
%
%   For more details of particular methods see their source m-files.
%
%------------------------------------------------------------------------
% .License.
%    Copyright (C) 2011  Charles University in Prague, Czech Republic
%
%   LIME 1.0 is free for private use and for purely academic purposes. 
%   It would be very kind from the future user of LIME 1.0 to give 
%   reference that this software package has been developed 
%   by at Charles University, Czech Republic.
%
%   For any other use of LIME 1.0 a license is required.
%
%   THIS SOFTWARE IS PROVIDED AS IS AND WITHOUT ANY EXPRESS OR IMPLIED 
%   WARRANTIES, INCLUDING, WITHOUT LIMITATIONS, THE IMPLIED WARRANTIES 
%   OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.   
%    
%-------------------------------------------------------------------------
% .History.
%       
%   2019-03-03    first version
%
%------------------------------------------------------------------------
% .Todo.
%
%  - todo diagonal and upper/lower triangular matrices
%  - rewrite pinv with verified full-column rank checking and inv
%  - vdet can fail, write a check  
%  
%ENDDOC===================================================================
  
  
[m, n] = size(iA);

state = 'vec';
method = 0;

if (m != n)
  error('idet: iA must be square');  
end
  
if (n==1) 
 id = iA(1,1);
 state='hull';
 return;
end 

if(n==2)
  id = iA(1,1)*iA(2,2) - iA(1,2)*iA(2,1);
  state = 'hull';
end


% preconditioning
Cinv = inv(mid(iA));  % rewrite !!!!
detCinv = vdet(Cinv);

iA = Cinv * iA;

% main computation
Arad = rad(iA);
Amag = mag(iA);

% if radii are relatively small to coefficient magnitudes use our Cramer' method
if (min(min(Arad)) < max(max(Amag)) / (10^5) )
  [id, state] = idetcram(iA);
  if( strcmp(state, 'vec'))
    id =  id / detCinv;
    method = 1;
    return;
  else
    warning('idet: Cramers method failed');  
  end
end

% otherwise continue with gaussian elimination
[id, state] = idetge(iA);
 if( strcmp(state, 'vec'))
    id = id / detCinv;
    method = 2;
    return;
  else
    warning('idet: Gaussian elimination method failed');  
  end


% if nothing works go with Hadamard's inequality and Gerschgorin
 warning('idet: switching to Hadamard inequality + Gerschgorin circles');  
 id1 = idethad(iA) / detCinv
 id2 = idetgersch(iA) / detCinv 
 
 id = intersect(id1, id2);
 
 method = 3;
 if( isfinite(inf(id)) && isfinite(sup(id)))
   state = 'vec';
 else
   state = 'inf';
 end
   
  
  


   
end