function [solution, unknown, discarded, depth] = ilsbab(initx, eps, iA, ib, typeofprune, residual = 0)  
%BEGINDOC=================================================================
% .Author.
%
%  Jaroslav Horacek
%
%-------------------------------------------------------------------------
% .Description.
%
%   Branch and Bound on solution
%
%   The function applies Branch and Bound approach on a interval linear 
%   systems Ax = b (solution). Using iterative pruning methods for classical 
%   interval systems.
%
%-------------------------------------------------------------------------
% .Input parameters.
%   
%   initx ... initial framing box before B&B procedure
%   vectors initial box(es) 
%   eps ... the minimum width of the interval that is allowed to divide
%   A ... real matrix  
%   b ... real column vector
%   typeofprune ... the string name of method of prunes for boxes (any method that
%                   returns [pruned box, is unfeasible, is verified] )
%
%------------------------------------------------------------------------
% .Output parameters.
%
%   solution ... boxes that may contain a solution
%   discarded ... boxes that certainly do not contain a solution 
%   depth ... determines the depth in which the box is discarded
%
%------------------------------------------------------------------------
% .Implementation details. 
%
%   The boxes we splits into two boxes according to longest side. 
%   boxes proved to contain at least some solution, are split again
%   boxes of small enough size containing at least one solution are marked as 
%   solution 
%
%------------------------------------------------------------------------
% .Licence.
%  
%   Copyright (C) 2018  Charles University in Prague, Czech Republic
%
%   LIME 1.0 is free for private use and for purely academic purposes. 
%   It would be very kind from the future user of LIME 1.0 to give 
%   reference that this software package has been developed 
%   by at Charles University, Czech Republic.
%
%   For any other use of LIME 1.0 a license is required.
%
%   THIS SOFTWARE IS PROVIDED AS IS AND WITHOUT ANY EXPRESS OR IMPLIED 
%   WARRANTIES, INCLUDING, WITHOUT LIMITATIONS, THE IMPLIED WARRANTIES 
%   OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.    
%    
%-------------------------------------------------------------------------
% .History.
%       
%   2018-01-02    first version
%   2018-11-05    using flags instead of state
%   2018-11-05    enabled differentiating between solution/unknown/discarded
%
%------------------------------------------------------------------------
% .Todo.
%   
%   - use feval to call pruning methods? 
%
%ENDDOC===================================================================

% initboxes fills the stack with some boxes (interval column vector) to prune
  [m, n] = size(initx);

  solution =  [];
  unknown = [];
  discarded = []; 
  depth = [];

  % column vectors
  stack = initx;
  splitdepth = zeros(n, 1);
  lastboxindex = n;
 
  
  if residual
     Ac = mid(iA);
     bc = mid(ib);
     x = inv(Ac * Ac') * Ac' * bc;  
     ib = ib -  Ac * x; 
  end  
  

 %  tic;

  while (lastboxindex > 0)
      %stack
      newbox = stack(:, lastboxindex);
      
%      if(any(isempty(newbox)))
%        lastboxindex = lastboxindex - 1;
%        continue;
%      end
      
      % HERE choose type of prune ****************************      
      %[newbox, unf] = prune_intersect(newbox, ipk, Ak, bk, A, b);
      % ******************************************************      
       
      [newbox, flag] = feval(typeofprune, newbox, iA, ib);
 
   
      %[newbox, unf, ver] = ilsprunejacobi(newbox, iA, ib);
      
   
      % newbox is verified to not contain solution 
      if (flag == 0)
          discarded = [discarded, stack(:,lastboxindex)];
          depth = [depth; splitdepth(lastboxindex)];
          lastboxindex = lastboxindex - 1;
          continue; 
      end       
   
      % whole newbox contains solutions
      if (flag == 2)
          solution = [solution, newbox];
          lastboxindex = lastboxindex - 1;
          continue; 
      end  
   
      % box too small and solution is probably there  
      if toosmalltest(newbox, eps)
           % row vectors!!!!     
           %solution = [solution, stack(:,lastboxindex)];
           if(flag == 1)
              solution = [solution, newbox];
           else           
              unknown = [unknown, newbox];
           end
           
           lastboxindex = lastboxindex - 1;
           continue;
      end    

     sd = splitdepth(lastboxindex); 
     
     [b1, b2] = splitbox(newbox);
     stack(:, lastboxindex) = b1;  
     stack(:, lastboxindex + 1) = b2;
     
     splitdepth(lastboxindex) = sd + 1;
     splitdepth(lastboxindex + 1) = sd + 1;
     
     lastboxindex = lastboxindex + 1;    
   
  end

 % toc;
 
   if residual
     Ac = mid(iA);
     bc = mid(ib);
     x = inv(Ac * Ac') * Ac' * bc;  
     
     len = size(solution, 2);
     if (len > 0)
        solution = solution + repmat(x, 1, len);
     end
   
     len = size(unknown, 2);
     if (len > 0)
        unknown = unknown + repmat(x, 1, len);
     end
     
     len = size(discarded, 2);
     if (len > 0)
        discarded = discarded + repmat(x, 1, len);
     end   
     
  end  
  
end

% tests whether all sides of box are smaller than eps
function tsm = toosmalltest(box, eps)
    
    tsm = 0;    
    diams = diam(box);
    
    if (max(diams) < eps)
        tsm = 1;
    end  
    
end

% splits the box into two boxes according to longest side
function [b1, b2] = splitbox(box)
    
    diams = diam(box);
    % finds index of maximal diameter
   
    %i = randi(length(diams));
    [c, i] = max(diams);
    
    b1 = box;
    b2 = box;
    
    splitint = box(i);
    
    b1(i) = infsup(inf(splitint), mid(splitint));
    b2(i) = infsup(mid(splitint), sup(splitint));
    
end