function [ix, state] = ilsshavingenc(iA, ib, ix, it=0, varindices=[1])
%BEGINDOC=================================================================
% .Author.
%
%  Jaroslav Horacek
%
%-------------------------------------------------------------------------
% .Description.
%
%   provides one step of interval Gauss-Seidel iteration method
%   works for square and overdetermined systems
%
%-------------------------------------------------------------------------
% .Input parameters.
%   
%   iA ...  m x n matrix, if the line is very long then we can
%          wrap it like this 
%   ib ...  n-dimensional right-hand side vector
%   it ... iterative improvement of shaving
%   varindices ... list of variable indices to be shaved from below and above
%
%------------------------------------------------------------------------
% .Output parameters.
%
%   ix ...  shaved ix 
%   state ... contains additional info 
%   'vec' if enclosure is finite or unbounded vector
%   'empty' system has no solution inside ix
%   'sin' ... system singular
%   'err' ... variable wchich coefficient is to be inflated is zero
%
%------------------------------------------------------------------------
% .Implementation details. 
%
%  Algorithm based on paper Hladik, Horacek, A Shaving Method
%   for Interval Linear Systems of Equations, 2014 
%
%------------------------------------------------------------------------
%  .Licence.
%  
%   Copyright (C) 2018  Charles University in Prague, Czech Republic
%
%   LIME 1.0 is free for private use and for purely academic purposes. 
%   It would be very kind from the future user of LIME 1.0 to give 
%   reference that this software package has been developed 
%   by at Charles University, Czech Republic.
%
%   For any other use of LIME 1.0 a license is required.
%
%   THIS SOFTWARE IS PROVIDED AS IS AND WITHOUT ANY EXPRESS OR IMPLIED 
%   WARRANTIES, INCLUDING, WITHOUT LIMITATIONS, THE IMPLIED WARRANTIES 
%   OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.    
%    
%-------------------------------------------------------------------------
% .History.
%       
%   2018-12-12      first version
%   2018-12-14      correction of indices
%
%------------------------------------------------------------------------
% .Todo.
%
%  -add checking if the variable from y,z related to coefficient to be inflated
%  is not zero? 
%
%ENDDOC===================================================================


% defined globally for convenience and reuse
[m, n] = size(iA);

if (isempty(varindices))
    varindices = [1:n];
end

for q = 1:length(varindices);
    
    varindex = varindices(q);
    %------------------------------------------
    %  shaving of a desired variable from above
    %------------------------------------------
    
    [iC, id, vari, varj, state] = formsystem(iA, ib, ix, varindex, 'sup');     
    if (strcmp(state, 'sha'))
       alpha = upperboundalpha(n, vari, varj, iC, id);
      
       % iterative improvement of alpha
       %if (it)
       %  
       %end
       
       ix(varindex) = infsupdec(inf(ix(varindex)), sup(ix(varindex)) - alpha);
    end

    %------------------------------------------
    %  shaving of a desired variable from below
    %------------------------------------------

    [iC, id, vari, varj, state] = formsystem(iA, ib, ix, varindex, 'inf');   
    if (strcmp(state, 'sha'))
       alpha = upperboundalpha(n, vari, varj, iC, id);
      
       % iterative improvement of alpha
       %if (it)
       %  
       %end
      
       ix(varindex) = infsupdec(inf(ix(varindex)) + alpha, sup(ix(varindex)));     
    end    


end


    
end



% function for bounding alpha from above ------------------------------------
function alpha = upperboundalpha(wno, i, j, iC, id)
        
       n = size(iC, 1);   
       iv = ilshbrenc(iC, id);
       vl = length(iv);
       
       iu = iv(wno+1:vl);
       if any(inf(iu) < 0)
         % sufficient condition is not satisfied
         alpha = 0;
         return;  
       end
   
       % i-th column of C^-1
       rs = zeros(n, 1);
       rs(i) = 1; 
       inviCi =  ilshbrenc(iC, infsupdec(rs));  
       
       % first upper bound on alpha
       alphabounds = [];
       if (inf(inviCi(j)) < 0)
          alphabounds = [alphabounds, -1/(inf(inviCi(j)))];  
       end
     
       % second upper bounds on alpha
      %K = find ( iv((wno+1):vl) >= 0 );
       %K = K + wno;
       
       for q=1:length(iu)
          k = q + wno;
          ib1 = sup(iv(j) * inviCi(k));
          ib2 = inf(iv(k)) * inf(inviCi(j));
                    
          if (ib1 > ib2) 
              alphabounds = [alphabounds, inf(iv(k)) / ( ib1 - ib2 ) ];  
          end
       end
     
       alphabounds
       if(length(alphabounds) >= 1 ) 
          alpha = min(alphabounds)
       else
          alpha = 0;
       end
end

% forms system iC x = id ------------------------------------------------------
function [iC, id, vari, varj, state] = formsystem(iA, ib, ix, varindex, boundtype)
  
  % coordinates (i,j) of variable that will be later inflated with alpha
  vari = 0;
  varj = 0;
  
  state = 'sha';     
  [m, n] = size(iA);  
  Ac = mid(iA);
  bc = mid(ib);
  I = eye(n);

  slicesup = sup(ix);
  sliceinf = inf(ix);
  
  if (strcmp(boundtype, 'sup'))
    sliceinf(varindex) = sup(ix(varindex));  
  else
    slicesup(varindex) = inf(ix(varindex));  
  end
  
  
  
  ctype = blanks(n);
  ctype(1:n) = 'S';
  lb = [-ones(1,n), zeros(1, 2*n)];
  ub = [ones(1,n), Inf(1, 2*n)];
  
   c = [bc', slicesup', -sliceinf'];
   C = [Ac', I, -I];
   d = zeros(n, 1);

    % computing minimum 
   [x, fmin, status, extra] = glpk(c, C, d, lb, ub, ctype);

   if (fmin >=0)
          iC = [];
          id = [];
          state = 'sol'; 
          return; 
   else
       bigI = [I, -I];
       lasteq = [slicesup', -sliceinf'];
       yz = x( (n+1):(3*n) );
       ilist = find(yz > eps); % find nonzero coefficients
       
       vari = n+1;
       
       % getting index of coefficient to be later inflated -----------------
       if (strcmp(boundtype, 'sup'))
          varj = n + find(ilist == (n+varindex));
       else
          varj = n + find(ilist == (varindex));
       end
       % -------------------------------------------------------------------
       
       selbigI = bigI(:, ilist);
       sellasteq = lasteq(ilist);
       
       iC = [iA', selbigI; ib', sellasteq];
       id = [infsupdec(zeros(n, 1)); -1];
       [newm, newn] = size(iC); 
       
       % Ac singular => ix does not bound the solution set
       if (newm > newn)
          state = 'sin';
          return;  
       end

       % if not square we need to complete the system to a square one      
       if(newm < newn)
           nullspace = transpose(null(mid(iC)));
           iC = [iC; nullspace];
           y = [x(1:n); x(ilist + n)];
           id = [id; nullspace * y ];
       end         
    end   
end

