function [ix, state, it, ver] = ilsgsenc(iA, ib, ioldx, e, maxit)
%BEGINDOC=================================================================
% .Author.
%
%  Jaroslav Horacek
%
%-------------------------------------------------------------------------
% .Description.
%
%   computes an enclosure of a system Ax=b
%   using Jacobi iterative method with preconditioning  
%   works for square and overdetermined systems
%
%-------------------------------------------------------------------------
% .Input parameters.
%   
%   A ...  m x n matrix, if the line is very long then we can
%          wrap it like this 
%   b ...  n-dimensional right-hand side vector
%   oldx ... initial enclosure of solution
%   e ... epsilon finish condition
%   maxit - max number of iterations (default 20)
%
%------------------------------------------------------------------------
% .Output parameters.
%
%   x ...  solution of system Ax = b
%   state ... contains additional info 
%   'vec' if enclosure is finite or unbounded vector
%   'empty' system has no solution if the estimation was correct
%   'exc' iterations exceeded
%   'zer' zeros on diagonal after preconditioning we cannot use this method
%   'est' - default Krawczyk estimation failed - estimation needs to be
%           given explicitely
%   'nan' - result of iteration contains nans
%   it ... number of iterations done 
%   ver ... 1 if the result is verified, 0 otherwise
%
%------------------------------------------------------------------------
% .Implementation details. 
%
%   we stop when the absolute differences between all infs and all sups of
%   the two consequent solutions are less than positive real number e 
%   (default min(min(diam(A)))/100 * 10^-5
%
%   uses initial krawczyk enclosure by Moore 
%
%------------------------------------------------------------------------
%  .Licence.
%  
%   Copyright (C) 2018  Charles University in Prague, Czech Republic
%
%   LIME 1.0 is free for private use and for purely academic purposes. 
%   It would be very kind from the future user of LIME 1.0 to give 
%   reference that this software package has been developed 
%   by at Charles University, Czech Republic.
%
%   For any other use of LIME 1.0 a license is required.
%
%   THIS SOFTWARE IS PROVIDED AS IS AND WITHOUT ANY EXPRESS OR IMPLIED 
%   WARRANTIES, INCLUDING, WITHOUT LIMITATIONS, THE IMPLIED WARRANTIES 
%   OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.    
%    
%-------------------------------------------------------------------------
% .History.
%       
%   2018-12-10      first version
%
%------------------------------------------------------------------------
% .Todo.
%   
%
%ENDDOC===================================================================

ver = 1;
state = 'vec';

[m, n] = size(iA);
%[iA,ib] = tointval(iA,ib);
it = 1;

ix = repmat(infsupdec(-Inf, Inf), n, 1);


% preconditioning of a system
C = inv(mid(iA));

iA = C * iA;
ib = C * ib;


% if the original system is overdetermined take only first n rows
if (m > n)
  iA = iA(1:n, 1:n);
  ib = ib(1:n);   
end

if (nargin < 3)
   % e chosen heurisctically
   e = min(min(diam(iA))) * 10^(-5);
   maxit = 20;  
  
   % simple estimation by Moore
   E = eye(n) - iA;
   Enorm = imaxnorm(E);
   if (Enorm >= 1)
      state = 'est';
      ver = 0;
      return; 
   end
   
   xi = infsupdec(-1,1) * imaxnorm(ib) / (1 - Enorm);
   ioldx = repmat(xi, n, 1); 
  
end

ix = ioldx;

% checking if a diagonal contains 0
% it can sometimes happen during preconditioning
if (any(in(0, diag(iA))))
  state = 'zer';
  return;
end


% main loop
while ( it <= maxit)
    
   % computing new estimations      
   [ix, state] = ilsgsstep(iA, ib, ioldx);
   if(strcmp(state, 'empty') || strcmp(state, 'nan'))
      return; 
   end
   

    % ending condition
    if ( all(abs(inf(ix)-inf(ioldx)) < e) && all(abs(sup(ix) - sup(ioldx)) < e ) )
        return;
    end

    ioldx = ix;
    
    it = it + 1;
end


% number of iterations exceeded
if( it > maxit )
    it = it - 1;
    state = 'exc';
end
    
end