function [ix, state] = ilsgsstep(iA, ib, ix)
%BEGINDOC=================================================================
% .Author.
%
%  Jaroslav Horacek
%
%-------------------------------------------------------------------------
% .Description.
%
%   provides one step of interval Gauss-Seidel iteration method
%   works for square and overdetermined systems
%
%-------------------------------------------------------------------------
% .Input parameters.
%   
%   iA ...  m x n matrix, if the line is very long then we can
%          wrap it like this 
%   ib ...  n-dimensional right-hand side vector
%   ix ... initial enclosure of solution
%
%------------------------------------------------------------------------
% .Output parameters.
%
%   ix ...  shaved ix using Jacobi iteration on iA x = ib
%   state ... contains additional info 
%   'vec' if enclosure is finite or unbounded vector
%   'empty' system has no solution inside ix
%   'nan' zeros on diagonal or too large intervals
%
%------------------------------------------------------------------------
% .Implementation details. 
%
%   if iA is rectangular m x n matrix, only first n rows are used for
%   iteration
%
%   !!! does not check nonzero diagonal
%   !!! does not provide preconditioning
%
%------------------------------------------------------------------------
%  .Licence.
%  
%   Copyright (C) 2018  Charles University in Prague, Czech Republic
%
%   LIME 1.0 is free for private use and for purely academic purposes. 
%   It would be very kind from the future user of LIME 1.0 to give 
%   reference that this software package has been developed 
%   by at Charles University, Czech Republic.
%
%   For any other use of LIME 1.0 a license is required.
%
%   THIS SOFTWARE IS PROVIDED AS IS AND WITHOUT ANY EXPRESS OR IMPLIED 
%   WARRANTIES, INCLUDING, WITHOUT LIMITATIONS, THE IMPLIED WARRANTIES 
%   OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.    
%    
%-------------------------------------------------------------------------
% .History.
%       
%   2015-05-11      first version
%   2017-05-08      deleted zero checking and preconditioning
%   2018-09-25      flags changed
%
%------------------------------------------------------------------------
% .Todo.
%  
%  Maybe skip checking tointval and zeros on diagonal to ensure speed
%
%ENDDOC===================================================================



state = 'vec';

[m, n] = size(iA);


% [iA,ib] = tointval(iA,ib);


%% checking if a diagonal contains 0
%for j=1:n
%   if( subset(0, iA(j, j) )  )
%       % zero on diagonal
%       state = 'zer';
%       return;
%   end
%end



% Gauss-Seidel iteration
 for i=1:n
        cols = [1:(i-1) (i+1):n ];
        ix0 = ( ib(i) - iA(i, cols ) * ix( cols ) ) / iA(i,i);        
        
        % intersect with old value at i-th position
        ix(i) = intersect( ix(i), ix0 );
        if( isempty(ix(i)) )
            % wrong first enclosure estimation
            state = 'empty';
            return;    
        end
        
        % checking whether new enclosure is finite 
        if (any (isnan(inf(ix(i)))) || any(isnan(sup(ix(i)))))
            state = 'nan';
            return; 
        end
 end

    
end
