function [ix, state, it, ver] = ilsjacobienc(iA, ib, e, maxit, ioldx)
%BEGINDOC=================================================================
% .Author.
%
%  Jaroslav Horacek
%
%-------------------------------------------------------------------------
% .Description.
%   
%   improved ilsjacobienc() only one preconditioning is needed
%
%   computes an enclosure of a system Ax=b
%   using Jacobi iterative method with preconditioning  
%   works for square and overdetermined systems
%
%-------------------------------------------------------------------------
% .Input parameters.
%   
%   A ...  m x n matrix, if the line is very long then we can
%          wrap it like this 
%   b ...  n-dimensional right-hand side vector
%   oldx ... initial enclosure of solution
%   e ... epsilon finish condition
%   maxit - max number of iterations (default 20)
%
%------------------------------------------------------------------------
% .Output parameters.
%
%   x ...  solution of system Ax = b
%   state ... contains additional info 
%   'vec' if enclosure is finite or unbounded vector
%   'empty' system has no solution if the estimation was correct
%   'exc' iterations exceeded
%   'zer' zeros on diagonal after preconditioning we cannot use this method
%   'est' - default Krawczyk estimation failed - estimation needs to be
%           given explicitely
%   'nan' - result of iteration contains nans
%   it ... number of iterations done 
%   ver ... 1 if the result is verified, 0 otherwise
%
%------------------------------------------------------------------------
% .Implementation details. 
%
%   implements using matrix multiplication, always uses preconditioning
%   as initial enclosure uses s very simple version of Krawczyk initial enclosure
%   Rohn. Introduction to interval analysis. 
%
%------------------------------------------------------------------------
%  .Licence.
%  
%   Copyright (C) 2018  Charles University in Prague, Czech Republic
%
%   LIME 1.0 is free for private use and for purely academic purposes. 
%   It would be very kind from the future user of LIME 1.0 to give 
%   reference that this software package has been developed 
%   by at Charles University, Czech Republic.
%
%   For any other use of LIME 1.0 a license is required.
%
%   THIS SOFTWARE IS PROVIDED AS IS AND WITHOUT ANY EXPRESS OR IMPLIED 
%   WARRANTIES, INCLUDING, WITHOUT LIMITATIONS, THE IMPLIED WARRANTIES 
%   OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.    
%    
%-------------------------------------------------------------------------
% .History.
%       
%   2018-09-28      first version
%   2018-12-10      preconditioning moved further
%   2018-12-17      works also for overdetermined systems
%
%------------------------------------------------------------------------
% .Todo.
%  - dealing with zeros on the main diagonal  
%  - make it work for m > n
%
%ENDDOC===================================================================

ver = 1;
state = 'vec';

[m, n] = size(iA);
%[iA,ib] = tointval(iA,ib);
it = 1;

ix = repmat(infsupdec(-Inf, Inf), n, 1);


% preconditioning of a system

%[iA, ib] = ilsprecondinv(iA, ib);

% if the original system is overdetermined take only first n rows
if (m > n)
  iA = iA(1:n, 1:n);
  ib = ib(1:n);   
end

if (nargin < 3)
   % e chosen heuristically
   e = min(min(diam(iA))) * 10^(-5);
   maxit = 20;  
end 

 
if(nargin < 5)  
   % simple estimation by Moore
   [ioldx, state] = initkrawczykenc(iA, ib)
%   E = eye(n) - iA;
%   Enorm = imaxnorm(E);
%   if (Enorm >= 1)
%      state = 'est';
%      ver = 0;
%      return; 
%   end
%   
%   xi = infsupdec(-1,1) * imaxnorm(ib) / (1 - Enorm);
%   ioldx = repmat(xi, n, 1); 
%  
end

ix = ioldx;


% checking if a diagonal contains 0
% it can sometimes happen during preconditioning
if (any(in(0, diag(iA))))
  state = 'zer';
  return;
end

iD = diag( 1./diag(iA) );

iJ = tril(iA, -1) + triu(iA, 1);



% main loop
while ( it <= maxit)
    
   % computing new estimations      
   ix = iD *(ib - iJ*ix);
   ix = intersect(ix, ioldx);
   
   
   
   if(any (isempty(ix)))
      state = 'empty';
      return; 
   end
   

    % ending condition - this takes less time than 
    % checking widths differences
    if ( all(abs(inf(ix)-inf(ioldx)) < e) && all(abs(sup(ix) - sup(ioldx)) < e ) )
        return;
    end

    ioldx = ix;
    
    it = it + 1;
end


% number of iterations exceeded
if( it > maxit )
    it = it - 1;
    state = 'exc';
end
    
end