function [ix, state] = ilsmagnitudeenc(iA, ib)
%-------------------------------------------------------------------------
% .Author.
%
%  Milan Hladik
%
%-------------------------------------------------------------------------
% .Description.
%
%   magnitude method for solving [iA]*x=[ib]
%
%-------------------------------------------------------------------------
% .Input parameters.
%   
%   iA ... n x n interval matrix, 
%   ib ... n-dimensional interval vector
%
%------------------------------------------------------------------------
% .Output parameters.
%
%   ix ... enclosure of the solution set of [iA]*x=[ib]
%        = NaN if [iA] is not stronly regular
%   state ... 'vec' if ix is an interval vector, 'dif' if conditions for 
%             algorithm are not met -> use different algorithm
%
%------------------------------------------------------------------------
% .Implementation details. 
%
%   Algorithm 3.1. described in the paper:
%   Milan Hladík. New operator and method for solving real preconditioned
%   interval linear equations. SIAM J. Numer. Anal., 52(1):194-206, 2014.
%
%------------------------------------------------------------------------
% .Licence.
%  
%   Copyright (C) 2018  Charles University in Prague, Czech Republic
%
%   LIME 1.0 is free for private use and for purely academic purposes. 
%   It would be very kind from the future user of LIME 1.0 to give 
%   reference that this software package has been developed 
%   by at Charles University, Czech Republic.
%
%   For any other use of LIME 1.0 a license is required.
%
%   THIS SOFTWARE IS PROVIDED AS IS AND WITHOUT ANY EXPRESS OR IMPLIED 
%   WARRANTIES, INCLUDING, WITHOUT LIMITATIONS, THE IMPLIED WARRANTIES 
%   OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.    
%    
%-------------------------------------------------------------------------
% .History.
%       
%   2015-07-01    first version
%   2018-02-09    \ substituted with vsol
%
%------------------------------------------------------------------------
% .Todo.
%   
%------------------------------------------------------------------------

state = 'vec';


[m, n] = size(iA);

if (m != n)
  error('ilsmagnitudeenc: iA must be a square matrix');  
end




%preconditioning
R = inv(mid(iA)); %preconditioning matrix
rA = infsupdec(mag(R*iA - eye(n))); %symmetrization such that R*iA<=[I-rA,I+rA]
ib = R*ib;

%check for strong regularity
v = mid(rA) * ones(n, 1);
if(~all(sup(rA*v) < v)) %simple sufficient condition failed
  v = vsol(abs((eye(n) - mid(rA)), ones(n,1))); %strong condition goes on
  if(~all(sup(rA*v) < v)) %strong condition failed
     ix = repmat(NaN, n, 1);
     state = 'dif'
     return;
  end
end

%step 1 of Algorithm 3.1. - computation of u
u = vsol(eye(n)-rA,mag(ib));

%step 2 of Algorithm 3.1. - estimation of the value d
d = diag(rA^2);
drA = diag(rA);
d = (1 + drA) ./ (1 - d);

gamma = ((1 - drA) - 1 ./ d);

%step 3 of Algorithm 3.1.
Da = 1 + (drA + gamma) * infsup(-1, 1); %denominator
rA2 = rA - diag(drA); %going to numerator...
ix = ib - rA2 * u * infsup(-1,1);

%the resulting enclosure
ix = infsupdec(sup(inf(ix) + gamma.*u), inf(sup(ix) - gamma .* u))./Da; 

end %of the main body
