function [iA, ib, C] = ilsprecondinv(iA, ib)
%BEGINDOC=================================================================
% .Author.
%
%  Jaroslav Horacek
%
%-------------------------------------------------------------------------
% .Description.
%
% Preconditions square interval linear system A x = b
% with approximate inverse of A_c (midpoint matrix of iA)
%
%-------------------------------------------------------------------------
% .Input parameters.
%   
%   iA ...  m x n matrix, if the line is very long then we can
%          wrap it like this 
%   ib ...  n-dimensional vector
%
%------------------------------------------------------------------------
% .Output parameters.
%
%   iA ...  preconditioned C
%   ib ...  preconditioned b
%
%------------------------------------------------------------------------
% .Implementation details. 
%
%   For overdetermined systems preconditioning is more complicated
%   we use matrix 
%   P =     | A_c1    0  | 
%           | A_c2 I(m-n)|
%   where A_c1 consists of first n rows of A_c, A_c2 is the rest
%   For square iA it is equivalent to preconditioning with Ac^-1
%  This procedure can be found in todo
%
%------------------------------------------------------------------------
%  .Licence.
%  
%   Copyright (C) 2018  Charles University in Prague, Czech Republic
%
%   LIME 1.0 is free for private use and for purely academic purposes. 
%   It would be very kind from the future user of LIME 1.0 to give 
%   reference that this software package has been developed 
%   by at Charles University, Czech Republic.
%
%   For any other use of LIME 1.0 a license is required.
%
%   THIS SOFTWARE IS PROVIDED AS IS AND WITHOUT ANY EXPRESS OR IMPLIED 
%   WARRANTIES, INCLUDING, WITHOUT LIMITATIONS, THE IMPLIED WARRANTIES 
%   OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.    
%    
%-------------------------------------------------------------------------
% .History.
%
%   2011-07-05      first version
%   2011-07-12      inv substituted by \
%   2011-07-29      preconditioning failure detection
%   2014-01-04      P is now real matrix
%   2015-04-24      applying template coding conventions  
%   2018-09-24      removed isnan checking and third output parameter state
%   2018-12-17      pinv substituted with inv (faster)
%
%------------------------------------------------------------------------
% .Todo.
%   
%   Here comes the list of future plans, if none leave empty 
%
%ENDDOC===================================================================

[m, n] = size(iA);
%[iA, ib] = tointval(iA,ib);

Ac = mid(iA);
Ac1 = Ac( 1:n, : );
Ac2 = Ac( (n+1:m), : );

O = zeros(n, m-n);
I = eye(m-n);

% precondition matrix
P = inv([ Ac1 O; Ac2 I ]);

iA = P * iA;
ib = P * ib;

% midpoint matrix is singular
%if( any(any(isnan(iA))) || any(isnan(ib)) )
%    state = 'fop';    
%end
    
end