function [iA, ib, ihull] = ilsexmpl(exmplname, n, p)
%BEGINDOC=================================================================
% .Author.
%
%  Jaroslav Horacek
%
%-------------------------------------------------------------------------
% .Description.
%
%  Set of examples of interval determinant matrices with its interval hull
%-------------------------------------------------------------------------
% .Input parameters.
%   
%   exmplname ... name of an example
%   n ... dimension
%   p ... parameter if necessary
%
%------------------------------------------------------------------------
% .Output parameters.
%
%   iA ... interval matrix
%   ib ... right-hand side vector
%   ihull ... hull of the interval determinant
%
%------------------------------------------------------------------------
% .Implementation details. 
%
%   In comments of each example we can found the source (usually we take not the original ones)
%   but the ones with visualisations
%   other verified determinants are computed using ilshull
%
%------------------------------------------------------------------------
% .Licence.
%  
%   Copyright (C) 2018  Charles University in Prague, Czech Republic
%
%   LIME 2.0 is free for private use and for purely academic purposes. 
%   It would be very kind from the future user of LIME 2.0 to give 
%   reference that this software package has been developed 
%   by at Charles University, Czech Republic.
%
%   For any other use of LIME 2.0 a license is required.
%
%   THIS SOFTWARE IS PROVIDED AS IS AND WITHOUT ANY EXPRESS OR IMPLIED 
%   WARRANTIES, INCLUDING, WITHOUT LIMITATIONS, THE IMPLIED WARRANTIES 
%   OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.    
%    
%-------------------------------------------------------------------------
% .History.
%       
%   2018-02-09    first version
%
%------------------------------------------------------------------------
% .Todo.
%   
%   Other examples. 
%
%ENDDOC===================================================================  
  
  
pkg load interval;

switch (exmplname)
  
%---------------------------------------------------------------------------  
% Hladik, Horacek, A shaving method for interval linear systems. 2013.
case 'hladikshaving'
  iA = infsup([-7 8; 5 -3 ], [-6 10; 6 -1]);
  ib = infsup([-11; -1], [-10; 1]);
  ihull =  infsup([-2.1579; -3.2632], [0; -1]);  
  ix0 = infsup([-2.1891; -3.2972], [1.0385; 0.1329]);
  
  
%---------------------------------------------------------------------------  
% A. Neumaier, Interval methods for systems of equations, 1990 - p. 93
case 'neumaier1'
  iA = infsup([-1 -1; 3 1], [7 -1; 3 1]);
  ib = infsup([1.2; -1.2]);
  ihull =  infsup([0.3; -0.6], [0.6; -0.3]);
  
  
%---------------------------------------------------------------------------  
% A. Neumaier, Interval methods for systems of equations, 1990 - p. 102
case 'neumaier2'
  iA = infsup([2 -1; -1 2], [2 0; 0 2]);
  ib = infsup([-1; -1], [1; 1]);
  ihull =  infsup([-1; -2], [1; 2]);  
 
 
%---------------------------------------------------------------------------  
% A. Neumaier, Interval methods for systems of equations, 1990 - p. 160
case 'neumaier3'
  iA = infsup(ones(n));
  iA = tril(iA);
  ib = infsup(zeros(n, 1));
  ib(1) = infsup(-p, p);
  ihull =  infsup(zeros(n, 1));
  ihull(1) = infsup(-p, p);
  ihull(2) = infsup(-p, p);  
  
 
%---------------------------------------------------------------------------  
% R. E. Moore, Introduction to interval analysis
case 'moore1'
  iA = infsup([2 -2; -1 2], [4 1; 2 4]);
  ib = infsup([-2; -2], [2; 2]);
  ihull =  infsup([-4; -4], [4; 4]);   
 
 
%---------------------------------------------------------------------------  
% Kraemer,Computing and Visualizing
%Solution Sets of Interval Linear Systems, 2006
case 'kraemer1'
  iA = infsup([2 -2; -1 2], [4 0; 0 4]);
  ib = infsup([1; -2], [2; 2]);
  ihull =  infsup([-1; -1.5], [4; 3]);    
  
 
%---------------------------------------------------------------------------  
% Kraemer,Computing and Visualizing
%Solution Sets of Interval Linear Systems, 2006
case 'kraemer2'
  iA = infsup([-2 1; 0 2], [-2 1.5; 6 3]);
  ib = infsup([-1; 1.5], [1; 3]);
  ihull = infsup([-0.25; 0.5], [1.625; 1.5]);    
  
%---------------------------------------------------------------------------  
% Kraemer,Computing and Visualizing
%Solution Sets of Interval Linear Systems, 2006
case 'kraemer3'
  iA = infsup([3.5 0 0 ; 0 3.5 0; 0 0 3.5], [3.5 2 2 ; 2 3.5 2; 2 2 3.5]);
  ib = infsup([-1; -1; -1], [1; 1; 1]);
  ihull =  "todo";      
  
%---------------------------------------------------------------------------    
% Skalna, Analysis of linear mechanical structures with uncertainties by
% means of interval methods, 1998.   
case 'skalna1'
  iA = infsupdec([0, 1; -2 -1], [1, 1; -2 2]);
  ib = infsupdec([0; 0 ], [2; 0]);  
  ihull = infsupdec([-2; 0], [2; 4]);
  initx = infsupdec([-3; -1], [3; 5]);
  
%---------------------------------------------------------------------------    
% Ning, Kearfott, A Comparison of Some Methods 
% for Solving Linear Interval Equations 1997.   
case 'ningkearfott1'
  iA = infsupdec(-ones(4), ones(4));
  iA(1,1) = infsupdec(4,6);
  iA(2,2) = infsupdec(-6,-4);
  iA(3,3) = infsupdec(9,11);
  iA(4,4) = infsupdec(-11,-9);
  ib = infsupdec([-2; 1; -4; 2], [4; 8; 10; 12]);  
  ihull = [];
  
%---------------------------------------------------------------------------    
% Ning, Kearfott, A Comparison of Some Methods 
% for Solving Linear Interval Equations 1997.   
case 'ningkearfott2'
  iA = infsupdec([3.7 -1.5 0; -1.5 3.7 -1.5; 0 -1.5 3.7], [4.3 -0.5 0; -0.5 4.3 -0.5; 0 -0.5 4.3]);
  ib = infsupdec([-14; -9; -3], [14; 9; 3]);  
  ihull = [];
     
%---------------------------------------------------------------------------    
% Ning, Kearfott, A Comparison of Some Methods 
% for Solving Linear Interval Equations 1997.   
case 'ningkearfott3'
  iA = infsupdec([3.7 -1.5 0; -1.5 3.7 -1.5; 0 -1.5 3.7], [4.3 -0.5 0; -0.5 4.3 -0.5; 0 -0.5 4.3]);
  ib = infsupdec([-14; -9; -3], [0; 0; 0]);  
  ihull = [];   
   
   
  otherwise
    disp('Error: example with that name does not exist') 
  
endswitch
  
  
end