function [fcr, ver, u, it] = ifcr(iA, iterate)
%BEGINDOC=================================================================
% .Author.
%
%  Jaroslav Horacek
%
%-------------------------------------------------------------------------
% .Description.
%
%   decides whether matrix iA has fcr (works even for real matrices)
%
%-------------------------------------------------------------------------
% .Input parameters.
%   
%   iA ...  interval or real matrix 
%   iterate ... if 1 (default) then iterative refinement is used
%
%------------------------------------------------------------------------
% .Output parameters.
%
%   fcr ...  1 if matrix has full column rank, 0 otherwise 
%   state ... 'fcr' for fcr, 'sin' matrix singular 
%   ver ... 1 if the result is verified
%   u ... returned u certifzing that  ||I-RA||_u < 1  
%
%------------------------------------------------------------------------
% .Implementation details. 
%
%   Checks whether || I - RA ||_u < 1, where R is pinv of Ac
%   if it is < 1 then the result is verified otherwise it is not
%------------------------------------------------------------------------
% .Licence.
%  
%   Copyright (C) 2018  Charles University in Prague, Czech Republic
%
%   LIME 1.0 is free for private use and for purely academic purposes. 
%   It would be very kind from the future user of LIME 1.0 to give 
%   reference that this software package has been developed 
%   by at Charles University, Czech Republic.
%
%   For any other use of LIME 1.0 a license is required.
%
%   THIS SOFTWARE IS PROVIDED AS IS AND WITHOUT ANY EXPRESS OR IMPLIED 
%   WARRANTIES, INCLUDING, WITHOUT LIMITATIONS, THE IMPLIED WARRANTIES 
%   OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.    
%    
%-------------------------------------------------------------------------
% .History.
%       
%   2018-03-19    first version
%   2018-08-02    iteration mistake corrected!!!
%   2019-04-09    improved computation of matrix R
%
%------------------------------------------------------------------------
% .Todo.
%
%ENDDOC===================================================================


ITNUM = 5;

if nargin < 2
  iterate =  1;
end
 

[m, n] = size(iA);

if (m < n)
  error('hasfcr: m < n case not implemented yet');  
end

if strcmp(typeinfo(iA), 'matrix')
    iA = midrad(iA, 10^-20);
end



if (m == n)
  R = inv(mid(iA));
  % If matrix is singular to machine precission Octave returns matrix full of Inf
  if (any(any(isinf(R))))
      fcr = 0;
      ver = 0;
      state = 'sin';
      return;
  end  
elseif (m > n)
  R = pinv(mid(iA));
    if (any(any(isnan(R))))
      fcr = 0;
      ver = 0;
      state = 'sin';
      return;
  end  
end

  

iC = eye(n) - R * iA;
 

u = ones(n, 1);

fcr = 0;
ver = 0;


fcr = imaxnorm(iC, u) < 1;


if fcr == 0 && iterate == 1

  it = 1;    
  while fcr == 0 && it <= ITNUM
    u = mag(iC) * u;
    fcr = imaxnorm(iC, u) < 1;
    it = it + 1;
  end

end


if fcr
   ver = 1; 
end


end