function [issin, state, ver] = issingular(iA)
%BEGINDOC=================================================================
% .Author.
%
%  Jaroslav Horacek
%
%-------------------------------------------------------------------------
% .Description.
%
%   decides whether matrix iA is singular
%
%-------------------------------------------------------------------------
% .Input parameters.
%   
%   iA ...  interval or real matrix 
%
%------------------------------------------------------------------------
% .Output parameters.
%
%   issin ...  1 if matrix is regular
%   state ... 'sin' if iA is singular, 'unk' if unknown
%   ver ... 1 if the result is verified, otherwise 0 
%
%------------------------------------------------------------------------
% .Implementation details. 
%
%  condition 1) is from J. Rohn. Systems of linear interval equations. 
%  Linear algebra and its applications,1989.
%
%  condition 2) is from J. Rohn. Checking properties of interval matrices. 
%  Technical Report 686, Institute of Computer Science, Academy of Sciences of 
%  the Czech Republic, Prague, 1996.
%
%------------------------------------------------------------------------
% .Licence.
%  
%   Copyright (C) 2018  Charles University in Prague, Czech Republic
%
%   LIME 1.0 is free for private use and for purely academic purposes. 
%   It would be very kind from the future user of LIME 1.0 to give 
%   reference that this software package has been developed 
%   by at Charles University, Czech Republic.
%
%   For any other use of LIME 1.0 a license is required.
%
%   THIS SOFTWARE IS PROVIDED AS IS AND WITHOUT ANY EXPRESS OR IMPLIED 
%   WARRANTIES, INCLUDING, WITHOUT LIMITATIONS, THE IMPLIED WARRANTIES 
%   OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.    
%    
%-------------------------------------------------------------------------
% .History.
%       
%   2019-03-31    first version
%
%------------------------------------------------------------------------
% .Todo.
%
%ENDDOC===================================================================

issin = 0;
state = 'unk';
ver = 0;

[m, n] = size(iA);

if (m != n)
  error('issingular: matrix must be n x n ');
end  

% test of the regularity of Ac
Ac = mid(iA);
Adelta = infsupdec(rad(iA));


% condition 1)
invAc = vinv(Ac);
C = mag(invAc) * Adelta;
dC = diag(mag(C));
if (max(dC) >= 1)
  issin = 1;
  ver = 1;
  state = 'sin';
  return;
end

% condition 2)

C = vinv(Adelta - abs(Ac));
if (all(all (C >= 0)))
  issin = 1;
  ver = 1;
  state = 'sin';  
end



end