function plotilssol(iA, ib, color='r', project)
%BEGINDOC=================================================================
% .Author.
%
%  Jaroslav Horacek
%
%-------------------------------------------------------------------------
% .Description.
%
%   Plots solution set of interval linear system
%
%
%-------------------------------------------------------------------------
% .Input parameters.
%   
%   iA ... matrix of system
%   ib ... right-hand side
%   color ... color of solution set area
%   project ...  if 1 (default), project solution set on only first 2 coordinates  
%
%------------------------------------------------------------------------
% .Output parameters.
%
%
%------------------------------------------------------------------------
% .Implementation details. 
%
% We rewrite Oettli-Prager theorem for each orthant, take all 
% the equations (boundaries of inequalities), compute all intersections, 
% check for these solutions, whether they satisfy all inequalities and compute 
% convex hull 
%
% projection on two first coordinates is due to handling of least squares s
% solution set
%
%------------------------------------------------------------------------
% .License.
%    Copyright (C) 2018  Charles University in Prague, Czech Republic
%
%   LIME 1.0 is free for private use and for purely academic purposes. 
%   It would be very kind from the future user of LIME 1.0 to give 
%   reference that this software package has been developed 
%   by at Charles University, Czech Republic.
%
%   For any other use of LIME 1.0 a license is required.
%
%   THIS SOFTWARE IS PROVIDED AS IS AND WITHOUT ANY EXPRESS OR IMPLIED 
%   WARRANTIES, INCLUDING, WITHOUT LIMITATIONS, THE IMPLIED WARRANTIES 
%   OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.   
%    
%-------------------------------------------------------------------------
% .History.
%       
%   2018-07-11 first version   
%   2018-10-01 corrected problem with rounding errors - TINYBIT added 
%   2018-10-15 projection added
%
%------------------------------------------------------------------------
% .Todo.
%   3D version - correct mistake - not enough points
%
%ENDDOC===================================================================


% ILS HULL!!!!!!!
[m, n] = size(iA);

% generate orthant signatures
Yn = generateyn(n);

ly = size(Yn, 1);


TINYBIT = 10^-6;

if (nargin < 4)
  project = 1;
end

if (nargin < 3)
  color = 'r';  
end

hold on;


% generate all combinations    
  clist = nchoosek(1:(2*m+2), 2); 
  lc = size(clist, 1); 


% for each orthant
for s = 1:ly

  Ty = diag( Yn(s, :) );


  % forming a system of inequalities for given orthant
  % + added axis
  
  
  A = [ mid(iA) - rad(iA)*Ty; -mid(iA) - rad(iA)*Ty; eye(n) ];
  b = [ rad(ib) + mid(ib); rad(ib) - mid(ib); repmat(0, n, 1)];
  
  [m1, n1] = size(A);
  
  
  sol = [];
  
  

  
  % for each pair of inequalities
  for i=1:lc
      pair = clist(i, :);
      
      subA = A(pair, :);
      subb = b(pair);
      
      % intersection of two boundaries      
      x = linsolve(subA, subb);
      
      
      % filtering points not lying in the desired orthant
      if (all ((Ty*x) >= -TINYBIT) )
      
        % checking if all inequalities are satisfied for x 
        leftside =  A(1:(2*m), :) * x;
        rightside = b(1:(2*m)) + TINYBIT;
        
        if ( all(leftside <= rightside) ) 
            sol = [sol, x];
        end

        
      end     
   
  end

 
  % sol
  
  % plotting the convex hull in the desired orthant
  if (!isempty(sol))
  
    % 2D
    if (n == 2 ) 
      x = sol(1, :);
      y = sol(2, :); 
    end  
      
    if (any(x) && any(y))  
      k = convhull(x, y);
      fill (x(k), y(k), color);
      plot(x(k), y(k), 'k')
    end
       
    
    % 3D
%    if ~project && (n == 3 ) 
%      x = sol(1, :);
%      y = sol(2, :); 
%      z = sol(3, :);
%      
%      sol
%      
%      if (any(x) && any(y) && any(z))  
%        k = convhulln(sol');
%        trisurf(k,x, y, z);
%        %fill (x(k), y(k), "r");
%        %plot(x(k), y(k), '-r')
%      end
%    end   
    
    

 
  end
 

end  

end

