function [ix, state, it] = oilsmultijacenc(iA, ib, overlap, e, maxit)
%BEGINDOC=================================================================
% .Author.
%
%  Jaroslav Horacek
%
%-------------------------------------------------------------------------
% .Description.
%
%   uses paralel Jacobi method on various subsquares chosen according to 
%   overlap parameter
%
%-------------------------------------------------------------------------
% .Input parameters.
%   
%   iA ... input system matrix 
%   ib ... right-hand side
%   overlap ... systems chosen to overlap by overlap 
%   e ... small positive number (default min(min(diam(A))) * 10^-5)
%   maxit ... safety stopping criterion (defalut 20)
%
%------------------------------------------------------------------------
% .Output parameters.
%
%   ix ... returned enclosure
%   state ... 'vec' or 'nof' not found, 'non' - no solution
%   it ... number of iterations 
%
%------------------------------------------------------------------------
% .Implementation details. 
%
%   Method explained in Horacek. Subsquares approach: a simple scheme for 
%   solving overdetermined interval systems. 2015
%
%------------------------------------------------------------------------
%  .Licence.
%  
%   Copyright (C) 2018  Charles University in Prague, Czech Republic
%
%   LIME 1.0 is free for private use and for purely academic purposes. 
%   It would be very kind from the future user of LIME 1.0 to give 
%   reference that this software package has been developed 
%   by at Charles University, Czech Republic.
%
%   For any other use of LIME 1.0 a license is required.
%
%   THIS SOFTWARE IS PROVIDED AS IS AND WITHOUT ANY EXPRESS OR IMPLIED 
%   WARRANTIES, INCLUDING, WITHOUT LIMITATIONS, THE IMPLIED WARRANTIES 
%   OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.    
%    
%-------------------------------------------------------------------------
% .History.
%       
%   2011-12-12 hypothetical version
%   2013-05-03 hypothetical new version enabling multicover by square
%               subsystems
%
%------------------------------------------------------------------------
% .Todo.
%   
%
%ENDDOC===================================================================

[m, n] = size(iA);

ix = repmat(infsup(-Inf, Inf), n, 1);
state = 'vec';
it = 0;


if ( (overlap >= n) | (overlap <= 0) )
    error('oilsjacsubsqenc: overlap must be 0 < overlap < n');
    return;
end
    
%[A,b] = tointval(A,b);

% default e, maxit
if( nargin < 5)
   e = min(min(diam(iA))) * 10^(-5);
   maxit = 20; 
end

% list of numbers of rows covered by some square subsystem
covered = [];
% list of numbers of rows waiting to be covered
waiting = [1:m];

% ==========================================
% choosing nxn subsystems
% ==========================================
middlesystemnum = floor( (m-n) / (n-overlap) );
remainingrows = mod(m-n, n-overlap);

systemnum = 1 + middlesystemnum + any(remainingrows);


iAmatrices = cell(systemnum);
ibmatrices = cell(systemnum);
iDs = cell(systemnum);
iJs = cell(systemnum);


% first system
indices = randnselfromlist( waiting, n);

covered = union(covered, indices)';
waiting = setdiff(waiting, indices);
iAmatrices{1} = iA(indices, :);
ibmatrices{1} = ib(indices);


% middle systems
for i=2:(middlesystemnum+1) 
    indices = randnselfromlist(waiting, n - overlap);
    indices = [indices, randnselfromlist(covered, overlap)];
    iAmatrices{i} = iA(indices, :);
    ibmatrices{i} = ib(indices);
    covered = [covered, indices];
    waiting = setdiff(waiting, indices);
    
end

% last system
indices = randnselfromlist(covered, n - remainingrows);
indices = [indices, waiting];
iAmatrices{systemnum} = iA(indices, :);
ibmatrices{systemnum} = ib(indices);

% ==========================================
% preconditioning of systems
% ==========================================
for i=1:systemnum 
    Ac = mid(iAmatrices{i});
    Acinv = inv(Ac);
    iAmatrices{i} = Acinv * iAmatrices{i};    
    ibmatrices{i} = Acinv * ibmatrices{i};    
end    

% ==========================================
% checking if a diagonals of preconditioned matrices contains 0
% it can sometimes happen during preconditioning
% + computing necessary iD and iJ
% ==========================================
for i=1:systemnum
    if( any ( in(0, diag( iAmatrices{i})) ))
        state = 'zer';
        return;
    end
    iDs{i} = diag( 1./diag(iAmatrices{i}) );
    iJs{i} = tril(iAmatrices{i}, -1) + triu(iAmatrices{i}, 1);

    
end


% ==========================================
% iterative hyperjacobi method
% ==========================================


% sofistikovanejsi hledani!!!!
%ix = ilshbrenc(iA(1:n, :), ib(1:n));
%ix = midsolest(A,b, 100);
ix = oilsrohnenc(iA, ib);
oldix = ix;


while ( it <= maxit)

    % does jacobi iteration for each subsquare system
    for sys=1:systemnum
        ix0 = iDs{sys} *(ibmatrices{sys} - iJs{sys}*ix);
               
        % intersect with old value at i-th position
        ix = intersect( ix, ix0 );
        if( any(isempty(ix)) )
            % wrong first enclosure estimation
            ix = repmat(NaN, n, 1);
            state = 'non';
            return;    
        end
    end

    % ending condition
    if ( all(abs(inf(ix)-inf(oldix)) < e) && all(abs(sup(ix) - sup(oldix)) < e ) )
        return;
    end
        
    oldix = ix;
    it = it + 1;
    
end


% maxit reached
if( it > maxit )
    it = it - 1;
    state = 'exc';
    ix = oldix;
end


end
