function [y, state, itnum] = oilsrohndit(G, g, e, maxit)

%BEGINDOC=================================================================
% .Author.
%
%  Jaroslav Horacek
%
%-------------------------------------------------------------------------
% .Description.
%
%   finds positive vector d satisfying Rohn's theorem
%
%-------------------------------------------------------------------------
% .Input parameters.
%   
%   G ...  matrix G from Rohn's theorem 
%   g ...  vector g from Rohn's theorem 
%   e ... small positive number (default min(min(diam(A))) * 10^(-5))
%   maxit defines maximal number of iteration steps
%
%------------------------------------------------------------------------
% .Output parameters.
%
%   y ... returned d
%   state ... 'vec' d found, 'exc' not found iterations exceeded, 'nof'
%              such d>0 not found
%   itnum ... number of iterations done
%
%------------------------------------------------------------------------
% .Implementation details. 
%
%   Gd + g < d
%   rewrites condition as 
%   d = Gd + g + e
%   with small positive e
%
%   if maximum of iterations given by param maxit is exceeded 
%   zero vector is returned
%
%------------------------------------------------------------------------
%  .Licence.
%  
%   Copyright (C) 2018  Charles University in Prague, Czech Republic
%
%   LIME 1.0 is free for private use and for purely academic purposes. 
%   It would be very kind from the future user of LIME 1.0 to give 
%   reference that this software package has been developed 
%   by at Charles University, Czech Republic.
%
%   For any other use of LIME 1.0 a license is required.
%
%   THIS SOFTWARE IS PROVIDED AS IS AND WITHOUT ANY EXPRESS OR IMPLIED 
%   WARRANTIES, INCLUDING, WITHOUT LIMITATIONS, THE IMPLIED WARRANTIES 
%   OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.    
%    
%-------------------------------------------------------------------------
% .History.
%       
%   2011-05-01    first version
%
%------------------------------------------------------------------------
% .Todo.
%   
%
%ENDDOC===================================================================

len = length( g );

% runs only when spectral radius of G
% is < 1

%if ( eigs(G, 1) >= 1)
   % disp('Error: spectral radius of G is >= 1');
%   y = zeros( len, 1 );
%   return;    
%end

% init to zero vector
state = 'vec';
newd = zeros( len, 1 );
d = repmat(e + 1, len, 1); % start condition to make while condition pass in first run

i = 1;

% repeats until |newd - d| < e
while ( ( i <= maxit ) && ( any ( abs( newd - d ) >= e ) )) 
    d = newd;
    newd = (G * d) + g + e; 
    i = i + 1;   
end

i = i - 1;
itnum = i;
    
% maximum of iterations exceeded 
% return zero vector
if( i > maxit )
    y = repmat(NaN, len, 1);
    state = 'exc';    
    return;
end

id = infsupdec(d);
if( any(id <= 0) )
    y = repmat(NaN, n, 1);
    state = 'nof';
    return;
end

y = d;
end