function [ix, state] = oilsrohnenc(iA, ib, cyclenum=1, iterate=0, e=10^-5, maxit=20)
%BEGINDOC=================================================================
% .Author.
%
%  Jaroslav Horacek
%
%-------------------------------------------------------------------------
% .Description.
%
%   returns an enclosure of a solution set of a system Ax = b using 
%   Rohn method
%
%-------------------------------------------------------------------------
% .Input parameters.
%   
%   iA ...  m x n matrix, if the line is very long then we can
%          wrap it like this 
%   ib ...  n-dimensional vector
%   iterate ... 0 (default )direct method used for finding d, 1 - iterative 
%      search for d 
%   e ... small positive number (default min(min(diam(A))) * 10^(-5))
%   maxit defines maximal number of iteration steps
%   cyclenum ... number of Rs chosen
%
%------------------------------------------------------------------------
% .Output parameters.
%
%   ix ... the enclosure of iAx=ib 
%   state ... a string flag
%       'vec' - x is interval vector
%       'dif' - if spectral radius of G is not < 1, use different method
%       'sin' - midpoint matrix might be singular
%       'exc' - iterations number exceeded
%       'nof' - direct method did not find d 
%
%------------------------------------------------------------------------
% .Implementation details. 
%
%   it always checks R = (Ac)^+
%------------------------------------------------------------------------
%  .Licence.
%  
%   Copyright (C) 2018  Charles University in Prague, Czech Republic
%
%   LIME 1.0 is free for private use and for purely academic purposes. 
%   It would be very kind from the future user of LIME 1.0 to give 
%   reference that this software package has been developed 
%   by at Charles University, Czech Republic.
%
%   For any other use of LIME 1.0 a license is required.
%
%   THIS SOFTWARE IS PROVIDED AS IS AND WITHOUT ANY EXPRESS OR IMPLIED 
%   WARRANTIES, INCLUDING, WITHOUT LIMITATIONS, THE IMPLIED WARRANTIES 
%   OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.    
%    
%-------------------------------------------------------------------------
% .History.
%       
%   2011-05-01    first version
%   2011-07-12    works for point real matrix
%   2015-05-11    applying template coding conventions 
%   2018-10-22    iterations added
%   2018-10-22    completely rewritten to incorporate iterations
%
%------------------------------------------------------------------------
% .Todo.
%  - vybrat jen jedno hledani d!   
%  - vraceni stavu
%  - kontrola fcr A !!!
%  - verifikovany check !!!
%
%ENDDOC===================================================================


[m, n] = size(iA);

%e = min(min(diam(iA))) * 10^(-5);
%maxit = 10;    

dfound = 0;

ix = repmat(infsupdec(-Inf, Inf), n, 1); 
state = 'dif';

Ac = mid(iA);
bc = mid(ib);
Delta = rad(iA);
delta = rad(ib);
I = eye(n);

% ========= generating of R matrices and x0 vectors ===============

Rs = cell(cyclenum); 
x0s = cell(cyclenum);

% 1) choices of matrice, first is R=Ac^+
Rs{1} = inv(Ac' * Ac) * Ac';
x0s{1} = Rs{1} * bc;

% 2) other matrices and vectors
for i=2:cyclenum
  A = randfim(iA);
  b = randfim(ib);
  
  % KONTROLA FCR !!!
  Rs{i} = inv(A' * A) * A';
  x0s{i} = Rs{i} * b;  
end


% ========== computing enclosures and their intersections ===========
for i=1:cyclenum
  R = Rs{i};
  x0 = x0s{i};
  G = mag( I - R * Ac ) + mag(R) * Delta;
  g = mag( R *( Ac * x0 - bc )) + mag(R) * (Delta * mag(x0) + delta);
  
  d = zeros(n, 1);
  state = 'vec';
  % neni treba checkovat?  !!!!
  %if (max(abs(eig(G))) < 1)
  if(iterate)  
      [d, state, itnum] = oilsrohndit(G, g, e, maxit);    
  else
      [d, state] = oilsrohnddir(G, g, e);     
  end
  
  
  if (strcmp(state, 'vec'))
    % VERIFIKOVANY check !!!    
    if( all(d > 0) && all(G*d + g < d)) 
%     plotboxes(midrad(x0, d)); 
      dfound = 1;
      ix = intersect( ix,  midrad(x0, d));
    end
  end

 
   %end  
  
end

%plotilssol(iA, ib);


% at least one d was found
if (dfound)
  state = 'vec';
else
  state = 'nof';
end
  
end