function solution = pilsbabonparams(initx, eps, ipk, Ak, bk, A, b )
%BEGINDOC=================================================================
% .Author.
%
%  Adam Szabo, Jaroslav Horacek
%
%-------------------------------------------------------------------------
% .Description.
%
%   Branch and Bound on solution
%
%   The function applies Branch and Bound approach on a parametric 
%   interval linear systems A(p)x = b(p) (parameters). Using iterative 
%   pruning methods for classical interval systems.
%
%-------------------------------------------------------------------------
% .Input parameters.
%   
%   initx ... initial framing box before B&B procedure
%   eps ... the minimum width of the interval that is allowed to divide
%   ipk ... column vector of intervals - bounds for parameters
%   Ak ... cellular array of arrays A{1}=[], A{2}=[]
%   bk ... cellular array of column vectors b{1}=, b{2}=
%   A ... real matrix  
%   b ... real column vector
%
%------------------------------------------------------------------------
% .Output parameters.
%
%   solution ... boxes that may contain a solution
%
%------------------------------------------------------------------------
% .Implementation details. 
%
%   The boxes we splits into two boxes according to longest side. 
%
%------------------------------------------------------------------------
% .Licence.
%  
%   Copyright (C) 2018  Charles University in Prague, Czech Republic
%
%   LIME 1.0 is free for private use and for purely academic purposes. 
%   It would be very kind from the future user of LIME 1.0 to give 
%   reference that this software package has been developed 
%   by at Charles University, Czech Republic.
%
%   For any other use of LIME 1.0 a license is required.
%
%   THIS SOFTWARE IS PROVIDED AS IS AND WITHOUT ANY EXPRESS OR IMPLIED 
%   WARRANTIES, INCLUDING, WITHOUT LIMITATIONS, THE IMPLIED WARRANTIES 
%   OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.    
%    
%-------------------------------------------------------------------------
% .History.
%       
%   2018-02-09    first version
%
%------------------------------------------------------------------------
% .Todo.
%   
%     
%
%ENDDOC===================================================================

n = length(ipk);
solution = [];

% column vectors
stack = [ipk];
lastboxindex = 1;

[iA, ib] = formmatrix(ipk, Ak, bk, A, b);
Precond = mid(iA); 
iA = Precond \ iA;
ib = Precond \ ib;


tic;

while (lastboxindex > 0)
    newbox = stack(:, lastboxindex); 
     
    
    % pruning by condition for unsolvability
 
    [iA, ib] = formmatrix(newbox, Ak, bk, A, b);
    [ns, ver] = isunsfcr(iA, ib); 
    

    if (ns)
        disp('ns');
        lastboxindex = lastboxindex - 1;
        continue; 
    end
    
     
    % box too small and zero probably there
    if toosmalltest(newbox, eps)
         % row vectors!!!!
         enc = iA\ib;
         
         % TODO prunik s initx
         % TODO test omezenosti
         solution = [solution, enc ];
         
         
         lastboxindex = lastboxindex - 1;
         continue;
    end    

    
   [b1, b2] = splitbox(newbox);
   stack(:, lastboxindex) = b1;  
   stack(:, lastboxindex + 1) = b2;
   lastboxindex = lastboxindex + 1;    
   
  
end

time = toc;

end


% creates interval matrix and vector from parametric systems 
function [iA, ib] = formmatrix(ipk, Ak, bk, A, b)
    
    %[m, n] = size(Ak{1});
    %iA = zeros(m, n);
    %ib = zeros(m, 1);
    
    iA = A;
    ib = b;

  if length(ipk) > 0    
    
    for i=1:length(ipk)
        iA = iA + ipk(i) .* Ak{i};
        ib = ib + ipk(i) .* bk{i};
    end
  end
  
  %  iA = iA + A;
  %  ib = ib + b;

end


% =======================================================================
function [newbox, unf] = prune_intersect(box, ipk, Ak, bk, A, b)
  
    [m, n] = size(Ak{1}); 
    testint = zeros(m, 1);
    paramnum = length(ipk); 
    unf = 1;
    
    for i=1:paramnum    
       testint = testint + ipk(i).*(Ak{i}*box - bk{i}) ;
    end
    testint = testint + (A*box - b);

    if all( in(0, testint) )
        unf = 0;
    end
    newbox = box;
    %unf=0;
 
end

%=========================================================================
function [newbox, unf] = prune_gs(box, iA, ib)
     
    unf = 0;
    [newbox, state] = ilsgsstep(iA, ib, box);
    if strcmp(state, 'non');
       unf = 1;  
    end 
     
end


%========================================================================
function [newbox, unf] = prune_hladik(box, ipk, Ak, bk, A, b)
  
    [m, n] = size(Ak{1});     
    leftside = zeros(m, 1);
    rightside = zeros(m, 1);
    
    paramnum = length(ipk); 
    unf = 0;
    
    for i=1:paramnum    
       leftside = leftside + mid(ipk(i)).*(Ak{i}*box - bk{i});
       rightside = rightside + rad(ipk(i)).*mag( (Ak{i}*box - bk{i}) );
    end
    
    leftside = mag(leftside);
    

    if any( leftside > rightside )
        unf = 1;
    end
    newbox = box;
    %unf=0;
 
end




% tests whether all sides of box are smaller than eps
function tsm = toosmalltest(box, eps)
    tsm = 0;    
    diams = wid(box);
    if (max(diams) < eps)
        tsm = 1;
    end  
end



% splits the box into two boxes according to longest side
function [b1, b2] = splitbox(box)
    diams = wid(box);
    
    % finds index of maximal diameter
    [c, i] = max(diams);
    
    b1 = box;
    b2 = box;
    
    splitint = box(i);
    
    b1(i) = infsup(inf(splitint), mid(splitint));
    b2(i) = infsup(mid(splitint), sup(splitint));
end

