function [solution, unknown, discarded, depth] = pilsbabonx(initx, eps, ipk, Ak, bk, A, b, typeofprune)  
%BEGINDOC=================================================================
% .Author.
%
%  Jaroslav Horacek
%
%-------------------------------------------------------------------------
% .Description.
%
%   Branch and Bound on solution
%
%   The function applies Branch and Bound approach on a interval linear 
%   systems Ax = b (solution). Using iterative pruning methods for classical 
%   interval systems.
%
%-------------------------------------------------------------------------
% .Input parameters.
%   
%   initx ... initial framing box before B&B procedure
%   vectors initial box(es) 
%   eps ... the minimum width of the interval that is allowed to divide
%   ipk ... column vector of intervals - bounds for parameters
%   Ak ... cellular array of arrays A{1}=[], A{2}=[]
%   bk ... cellular array of column vectors b{1}=, b{2}=
%   A ... real matrix  
%   b ... real column vector
%   typeofprune ... string with the name of method of prunes for boxes
%
%------------------------------------------------------------------------
% .Output parameters.
%
%   solution ... boxes that may contain a solution
%   discarded ... boxes that certainly do not contain a solution 
%   depth ... determines the depth in which the box is discarded
%
%------------------------------------------------------------------------
% .Implementation details. 
%
%   The boxes we splits into two boxes according to longest side. 
%
%------------------------------------------------------------------------
% .Licence.
%  
%   Copyright (C) 2018  Charles University in Prague, Czech Republic
%
%   LIME 1.0 is free for private use and for purely academic purposes. 
%   It would be very kind from the future user of LIME 1.0 to give 
%   reference that this software package has been developed 
%   by at Charles University, Czech Republic.
%
%   For any other use of LIME 1.0 a license is required.
%
%   THIS SOFTWARE IS PROVIDED AS IS AND WITHOUT ANY EXPRESS OR IMPLIED 
%   WARRANTIES, INCLUDING, WITHOUT LIMITATIONS, THE IMPLIED WARRANTIES 
%   OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.    
%    
%-------------------------------------------------------------------------
% .History.
%       
%   2018-01-02    first version
%   2018-12-22    feval is used, rewritten to match ilsbab 
%
%------------------------------------------------------------------------
% .Todo.
%   
%   Added other methods of prunes for boxes.
%   Change the eighth input parameter (typeofprune) to the parameter of 
%   the function.   
%
%ENDDOC===================================================================

  [m, n] = size(initx);

  solution =  [];
  unknown = [];
  discarded = []; 
  depth = [];
  
  % column vectors
  stack = initx;
  splitdepth = zeros(n, 1);
  lastboxindex = n;


%  tic;

  while (lastboxindex > 0)
      %stack
      newbox = stack(:, lastboxindex);
      

      
      % HERE choose type of prune ****************************      
      %[newbox, unf] = prune_intersect(newbox, ipk, Ak, bk, A, b);
      % ******************************************************      
       
      %[newbox, flag] = feval(typeofprune, newbox, ipk, Ak, bk, A, b);   
      
      [newbox, flag] = pilsprunehladik(newbox, ipk, Ak, bk, A, b);   
   
      % newbox is verified to not contain solution 
      if (flag == 0)
          discarded = [discarded, stack(:,lastboxindex)];
          depth = [depth; splitdepth(lastboxindex)];
          lastboxindex = lastboxindex - 1;
          continue; 
      end       
   
   
   
      [newbox, flag] = pilspruneoettli(newbox, ipk, Ak, bk, A, b);   
   
      % newbox is verified to not contain solution 
      if (flag == 0)
          discarded = [discarded, stack(:,lastboxindex)];
          depth = [depth; splitdepth(lastboxindex)];
          lastboxindex = lastboxindex - 1;
          continue; 
      end       
   
   
   
      % whole newbox contains solutions
      if (flag == 2)
          solution = [solution, newbox];
          lastboxindex = lastboxindex - 1;
          continue; 
      end  
   
      % box too small and solution is probably there  
      if toosmalltest(newbox, eps)
           % row vectors!!!!     
           %solution = [solution, stack(:,lastboxindex)];
           if(flag == 1)
              solution = [solution, newbox];
           else           
              unknown = [unknown, newbox];
           end
           
           lastboxindex = lastboxindex - 1;
           continue;
      end    

     sd = splitdepth(lastboxindex); 
     
     [b1, b2] = splitbox(newbox);
     stack(:, lastboxindex) = b1;  
     stack(:, lastboxindex + 1) = b2;
     
     splitdepth(lastboxindex) = sd + 1;
     splitdepth(lastboxindex + 1) = sd + 1;
     
     lastboxindex = lastboxindex + 1;    
   
  end
  
 % toc
  

end

% creates interval matrix and vector from parametric systems 
function [iA, ib] = formmatrix(ipk, Ak, bk, A, b)
    
    iA = A;
    ib = b;
    
    if length(ipk) > 0
    
      for i=1:length(ipk)
        iA = iA + ipk(i) .* Ak{i};
        ib = ib + ipk(i) .* bk{i};
      end
    
    end
 
end



% tests whether all sides of box are smaller than eps
function tsm = toosmalltest(box, eps)
    
    tsm = 0;    
    diams = diam(box);
    
    if (max(diams) < eps)
        tsm = 1;
    end  
    
end

% splits the box into two boxes according to longest side
function [b1, b2] = splitbox(box)
    
    diams = diam(box);
    % finds index of maximal diameter
   
    %i = randi(length(diams));
    [c, i] = max(diams);
    
    b1 = box;
    b2 = box;
    
    splitint = box(i);
    
    b1(i) = infsup(inf(splitint), mid(splitint));
    b2(i) = infsup(mid(splitint), sup(splitint));
    
end